using System;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class ControlUtil : IDisposable {
	    /// <summary>Interface that must be supported by all control imbedding this class</summary>
	    public interface IRepControl {
	        /// <summary>Rep++ is reinitialized in design mode</summary>
	        void            RepPPReInit();
            /// <summary>Rep++ is reset in design mode</summary>
            void            RepPPReset();
	    }
	    
	    /// <summary>Parent WEB control</summary>
	    private object                  m_parent;
	    /// <summary>Rep++ Application object</summary>
	    private Application             m_app;
	    /// <summary>Must use the Resource Dispenser only if true</summary>
	    private bool                    m_bResDispenserOnly;
	    /// <summary>true if application holder reference counting is disabled</summary>
	    private bool                    m_bNoRefCount;
	    /// <summary>Was unable to get the application object. Wait for Rep++ Reinitialization to try again</summary>
	    private bool                    m_bInError;
        /// <summary>true if control registred to the InitNotification</summary>
        private bool                    m_bRegistred;
        /// <summary>Cookie return at the registration</summary>
        private object                  m_cookieRegistration;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="parent">           Parent Web control</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        /// <param name="bNoRefCount">      true if parent cannot always call dispose. In this case, no notification will be sent when Rep++ is reinitialized</param>
        //
        //********************************************************************************
        public ControlUtil(object parent, bool bResDispenserOnly, bool bNoRefCount) {
            m_parent                = parent;
		    m_app                   = null;
		    m_bResDispenserOnly     = bResDispenserOnly;
		    m_bNoRefCount           = bNoRefCount;
            m_bInError              = false;
            m_bRegistred            = false;
            m_cookieRegistration    = null;
		}

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="parent">           Parent Web control</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public ControlUtil(object parent, bool bResDispenserOnly) : this(parent, bResDispenserOnly, false) {
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                Dispose the object
        /// </summary>
        //
        //********************************************************************************
        public virtual void Dispose() {
            RepPP.InitNotificationEventHandler  delInit;
            RepPP.InitNotificationEventHandler  delReset;

		    if (m_app != null) {
		        if (!m_bNoRefCount) {
		            m_app.Release();
		        }
		        m_app = null;
		    }
            if (m_bRegistred) {
                delInit  = new RepPP.InitNotificationEventHandler(this.RepPPInitialized);
                delReset = new RepPP.InitNotificationEventHandler(this.RepPPResetting);
                InitNotification.UnregisterControl(delInit, delReset, m_cookieRegistration);
                m_bRegistred         = false;
                m_cookieRegistration = null;
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// Site:               Returns the site object
		/// </summary>
        //
        //********************************************************************************
		private ISite Site {
		    get {
		        ISite                                   siteRetVal;
		        System.Web.UI.WebControls.WebControl    webCtl;
		        
		        if (m_parent is System.Web.UI.WebControls.WebControl) {
		            webCtl      = (System.Web.UI.WebControls.WebControl)m_parent;
		            siteRetVal  = webCtl.Site;
		            if (siteRetVal == null && webCtl.Page != null) {
		                siteRetVal = webCtl.Page.Site;
		            }
                } else if (m_parent is System.ComponentModel.IComponent) {
		            siteRetVal = ((System.ComponentModel.IComponent)m_parent).Site;
		        } else {
		            siteRetVal = null;
		        }
		        return(siteRetVal);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Parent:                     Parent control
		/// </summary>
        //
        //********************************************************************************
		protected object Parent {
            get {
                return(m_parent);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// IsDesignMode:               Indicates if the control is in design mode
		/// </summary>
        //
        //********************************************************************************
		internal virtual bool IsDesignMode {
            get {
#if !VC8
		        ISite   site;
#endif
                bool bRetVal;

                if (m_parent != null && m_parent is System.Web.UI.WebControls.WebControl) {
                    bRetVal = (System.Web.HttpContext.Current == null);
                } else {
#if VC8
                    bRetVal = (System.Diagnostics.Process.GetCurrentProcess().ProcessName == "devenv");
#else
                    site    = Site;
                    bRetVal = (site != null && site.DesignMode == true);
#endif
                }
		        return (bRetVal);
            }
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPInitialized:           Called when the Rep++ is initialized. Called only in design mode.
		/// </summary>
        //
        //********************************************************************************
        public virtual void RepPPInitialized(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                m_bInError = false;
                if (m_parent is IRepControl) {
                    ((IRepControl)m_parent).RepPPReInit();
                }
                if (m_parent is System.Web.UI.WebControls.WebControl) {
                    UpdateDesignTimeHtml();
                }
                break;
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPResetting:             Called when the Rep++ is reset. Called only in design mode.
		/// </summary>
        //
        //********************************************************************************
        public virtual void RepPPResetting(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                if (m_parent is IRepControl) {
                    ((IRepControl)m_parent).RepPPReset();
                }
                break;
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// Application:            Returns the Rep++ application object
		/// </summary>
        //
        //********************************************************************************
		public Application Application {
		    get {
                RepPP.Application                   appRetVal = null;
                RepPP.InitNotificationEventHandler  delInit;
                RepPP.InitNotificationEventHandler  delReset;
		        bool                                bDesignMode;
                ISite                               site;
		        
		        if (m_app != null) {
                    appRetVal = m_app;
                } else {
		            bDesignMode = IsDesignMode;
                    if (bDesignMode) {
                        site = Site;
                        //To speed the designer loading ...we wait until the site becomes available
                        // and only then we register.
                        if (!m_bRegistred && site != null) {
                            delInit              = new RepPP.InitNotificationEventHandler(this.RepPPInitialized);
                            delReset             = new RepPP.InitNotificationEventHandler(this.RepPPResetting);
                            m_cookieRegistration = InitNotification.RegisterControl(delInit, delReset, site);
                            m_bRegistred = true;
                        }
                        if (!m_bInError) {
                            try {
                                appRetVal  = RepPP.Application.GetApplicationForDesign(Site);
                            } finally {
                                m_bInError = (appRetVal == null);
                            }
                        }
                    } else {
		                if (m_bNoRefCount) {
		                    appRetVal  = Application.GetApplication();
		                } else {
		                    appRetVal = Application.CreateFromRes(LanguageCode.sdDefault, null);
		                    if (appRetVal != null && m_bResDispenserOnly && appRetVal.SessionId == -1) {
                                appRetVal.Release();
    	                        throw new ApplicationException("Rep++ must be started using the Resource Dispenser. Change the connection setting.");
    		                }
    		            }
                        m_app = appRetVal;
                    }
    		    }
		        return(appRetVal);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// UpdateDesignTimeHtml:           Updates the design-time HTML
		/// </summary>
		//
        //********************************************************************************
        protected virtual void UpdateDesignTimeHtml() {
            IDesignerHost   designerHost;
            ControlDesigner ctlDesigner;
            ISite           site;

            try {
                site = Site;
                if (site != null) {
                    designerHost = site.GetService(typeof(IDesignerHost)) as IDesignerHost;
                    if (designerHost != null) {
                        ctlDesigner = designerHost.GetDesigner(m_parent as System.Web.UI.WebControls.WebControl) as ControlDesigner;
                        if (ctlDesigner != null) {
                            ctlDesigner.UpdateDesignTimeHtml();
                        }
                    }
                }
            }
            catch {
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            IComponentChangeService     changeService;
            ISite                       site;

            site = Site;
            if (site != null) {
                changeService =  site.GetService(typeof(IComponentChangeService)) as IComponentChangeService;
                if (changeService != null) {
                    changeService.OnComponentChanging(m_parent, null);
                    changeService.OnComponentChanged(m_parent, null, "", "");
                }
            }
        }
    } // Class
} // Namespace
