using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using sdfwwebcsnet;
using System.Web;
using System.IO;
using RepPP;

namespace RepPP.VSNetWizard.Web
{
	/// <summary>
	/// SCGeneratorBase: Base class for application or item generation
	/// </summary>
	internal abstract class GeneratorBase
	{
        //*********************************************************     
        //
        /// <summary>
        /// CreateProjectp:                 Create a new visual studio project
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <returns>
        /// The created project
        /// </returns>
        //  
        //*********************************************************     
        protected abstract EnvDTE.Project CreateProjectP(ref string strPrjName, ref string strPrjFullName);

        //*********************************************************     
        //
        /// <summary>
        /// AddDispatchClassv:          Add the dispatch class to the project 
        /// </summary>
        /// <param name="strPrjName">   Project name</param>
        //
        //*********************************************************     
        protected abstract void AddDispatchClassV(string strPrjName);

        //*********************************************************     
        //
        /// <summary>
        /// GenCardTypedInstanceV:          Genearte card typed instance 
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Card name</param>
        //
        //*********************************************************     
        protected abstract void GenCardTypedInstanceV(string strPrjName, string strCardName);

        //*********************************************************     
        //
        /// <summary>
        /// GenGroupTypedInstanceV:         Generate group typed instance 
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strGroupName">     Group name</param>
        //
        //*********************************************************     
        protected abstract void GenGroupTypedInstanceV(string strPrjName, string strGroupName);

        //*********************************************************     
        //
        /// <summary>
        /// CodeFileExtension:         Get the extension of the code file
        /// </summary>
        //
        //*********************************************************     
        protected abstract string CodeFileExtension { get; }

        public const string IS_HTML_EXIST_CMD_SQL = "SELECT COUNT(*)FROM MODHTMLFILE" + 
                                                    " WHERE APPSYSTNAME = :APPSYSTNAME" +
                                                    " AND SECTFULLPATH = :SECTFULLPATH" +
                                                    " AND MODNAME = :MODNAME " +
                                                    " AND HTMLFILENAME = :HTMLFILENAME";
        public const string INSERT_HTML_CMD_SQL   = "INSERT INTO MODHTMLFILE" +
                                                    "(APPSYSTNAME, SECTFULLPATH, MODNAME, HTMLFILENAMEUC, HTMLFILENAME, HTMLOPT, ISMULTILANGUAGE, HTMLFILEDESC, ATTRSVAL)" +
                                                    " VALUES(:APPSYSTNAME, :SECTFULLPATH, :MODNAME, :HTMLFILENAMEUC, :HTMLFILENAME, :HTMLOPT, :ISMULTILANGUAGE, :HTMLFILEDESC, :ATTRSVAL)";

        protected EnvDTE.DTE                      m_vsApp;                      // Visual studio instance object
        protected RepPP.IApplication              m_app;                        // Rep++ application object
        protected string                          m_strHTMLTemplateDir;         // HTML Templates diretory
        protected string                          m_strHTMLMetaTemplateFile;    // The HTML meta-template file
        protected ArrayList                       m_arrLanguages;               // selected language for the applicaion
        protected Utility.GenTypedInstParam       m_genTypedInstParam;          // Typed instances generation parameters

        //******************************************************************
        //
        /// <summary>
        /// ctor:                                   Class constructor
        /// </summary>
        /// <param name="vsApp">                    Visual studio application</param>
        /// <param name="app">                      Rep++ application object</param>
        /// <param name="strHTMLTemplateDir">       HTML templates diretory</param>
        /// <param name="strHTMLMetaTemplateFile">  HTML metat-emplate file</param>
        /// <param name="arrLanguages">             Selected languages</param>
        /// <param name="genTypedInstParam">        Typed instance generation parameters</param>
        //
        //******************************************************************
		public GeneratorBase(EnvDTE.DTE                 vsApp,
                             RepPP.IApplication         app,
                             string                     strHTMLTemplateDir,
                             string                     strHTMLMetaTemplateFile,
                             ArrayList                  arrLanguages,
                             Utility.GenTypedInstParam  genTypedInstParam)
		{
            m_vsApp                     = vsApp;
            m_app                       = app;
            m_strHTMLTemplateDir        = strHTMLTemplateDir;
            m_strHTMLMetaTemplateFile   = strHTMLMetaTemplateFile;
            m_arrLanguages              = arrLanguages;
            m_genTypedInstParam         = genTypedInstParam;
        }

        //******************************************************************
        //
        /// <summary>
        /// InternalGeneratev:          Internl function for HTML form generation
        /// </summary>
        /// <param name="strPrjName">   Project name</param>
        /// <param name="strCardName">  Card name</param>
        //
        //******************************************************************
        private void InternalGeneratev(string strPrjName, string strCardName)
        {
            MySCGenTemplate     genTemplate;
            UserDispatch        disp;
            SCParameters        param;
            EnvDTE.Project      project;

            project     = GetProjectP(strPrjName);
            genTemplate = new MySCGenTemplate(project, m_app, strCardName);
            disp        = new UserDispatch(m_app, Path.GetDirectoryName(project.FullName));
            param       = new SCParameters(disp.Trace, disp);
            param.InitializeFromParam(SCDispatch.IDAppName + "=" + strPrjName);
            param.InitializeFromParam("_template=" + m_strHTMLMetaTemplateFile);
            param.InitializeFromParam(SCDispatch.IDCard + "=" + strCardName);
            genTemplate.Execute(disp, param, false);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateApplicationV:           Create a new web application
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <param name="strCardName">      Card name</param>
        /// <param name="strSBCardName">    Selection buffer card name</param>
        //
        //******************************************************************
        public void GenerateApplicationV(ref string     strPrjName, 
                                         ref string     strPrjFullName,
                                         string         strCardName,
                                         string         strSBCardName) {
            CreateProjectP(ref strPrjName, ref strPrjFullName);
            if (m_genTypedInstParam.m_strNamespace == String.Empty) {
                m_genTypedInstParam.m_strNamespace = strPrjName;
            }
            InitProject(strPrjName);
            AddDispatchClassV(strPrjName);
            UpdateConfigFilev(strPrjName);
            InternalGeneratev(strPrjName, strCardName);
            if (strSBCardName != null && strSBCardName != String.Empty) {
                GenListViewPagev(strPrjName,
                                 strCardName,
                                 strSBCardName);
            }
            GenFrameworkXmlFilev(strPrjName);
            // Copy templates in the project directory
            if (Directory.Exists(m_strHTMLTemplateDir)) {
                Utility.CopyDirectory(m_strHTMLTemplateDir, Path.GetDirectoryName(strPrjFullName));
            }
            if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                GenCardTypedInstanceV(strPrjName, strCardName);
                if (strSBCardName != null && strSBCardName != String.Empty) {
                    GenCardTypedInstanceV(strPrjName, strSBCardName);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateCardItemV:              Create card item (HTML forms)
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Card name</param>
        /// <param name="strSBCardName">    Selection buffer card name</param>
        //
        //******************************************************************
        public void GenerateCardItemV(string    strPrjName, 
                                      string    strCardName,
                                      string    strSBCardName) {
            if (m_genTypedInstParam.m_strNamespace == String.Empty) {
                m_genTypedInstParam.m_strNamespace = strPrjName;
            }
            InternalGeneratev(strPrjName, strCardName);
            if (strSBCardName != null && strSBCardName != String.Empty) {
                GenListViewPagev(strPrjName,
                                 strCardName,
                                 strSBCardName);
            }
            if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                GenCardTypedInstanceV(strPrjName, strCardName);
                if (strSBCardName != null && strSBCardName != String.Empty) {
                    GenCardTypedInstanceV(strPrjName, strSBCardName);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateGroupItemV:         Generate an HTML form for a group
        /// </summary>
        /// <param name="strPrjName">   Project name</param>
        /// <param name="strGroupName"> Group name</param>
        //
        //******************************************************************
        public void GenerateGroupItemV(string strPrjName, string strGroupName) {
            RepPP.Card  crd = null;
            string      strCardName;

            try {
                if (m_genTypedInstParam.m_strNamespace == String.Empty) {
                    m_genTypedInstParam.m_strNamespace = strPrjName;
                }
                strCardName = Utility.GetNotDefinedCardName(m_app);
                crd         = m_app.Cards.Add(strCardName);
                crd.ChildCardGroups.Add(m_app.Groups[strGroupName]);
                InternalGeneratev(strPrjName, strCardName);
                if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                    GenGroupTypedInstanceV(strPrjName, strGroupName);
                }
            } finally {
                if (crd != null) {
                    crd.Delete();
                }
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// GenListViewPagev:               Generate the listview page
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      principal card name</param>
        /// <param name="strSBCardName">    Selection buffer card name</param>
        //  
        //*********************************************************     
        private void GenListViewPagev(string    strPrjName,
                                      string    strCardName,
                                      string    strSBCardName) {
            RepPP.Card              crd;
            RepPP.Card              crdsb;
            string                  strFileName;
            StreamWriter            writer;
            string                  strText;
            EnvDTE.Project          project;

            project     = GetProjectP(strPrjName);
            crd         = m_app.Cards[strCardName];
            crdsb       = m_app.Cards[strSBCardName];
            strFileName = Path.GetDirectoryName(project.FullName);
            strFileName = Path.Combine(strFileName, "index" + strCardName + ".html");
			using (writer = Utility.CreateTextFile(strFileName, Utility.ReplaceFileModeE.sdReplaceWithoutAsk)) {
                strText = "<html>" + "\r\n" +
	                      "    <head>" + "\r\n" +
		                  "        <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">" + "\r\n" +
	                      "    </head>" + "\r\n" +
	 	                  "    <frameset cols=\"250,*\" style=\"border: thin solid #4A5A7B\">" + "\r\n" +
		 	              "        <frame name=\"FRMLISTVIEW\" src=\"dispatch.rem?_appname=" +  strPrjName +
                                                                                      "&_card=" + crdsb.Name + "." + crdsb.ChildCardGroups[0].Group.Name +
                                                                                      "&_lvcard=" + crd.Name + "." + crd.ChildCardGroups[0].Group.Name +
                                                                                      "&_wo=FILLLISTVIEW()&_pk=&_lvtarget=FRMMAIN\" marginwidth=\"0\" marginheight=\"0\" scrolling=\"yes\" unselectable=\"on\">" + "\r\n" +
			              "        <frame name=\"FRMMAIN\" unselectable=\"on\">" + "\r\n" +
	 	                  "    </frameset>" + "\r\n" +
	                      "    <noframes>" + "\r\n" +
		                  "        Votre navigateur doit supporter les 'frames'." + "\r\n" +
		                  "        Your navigator must support Frames." + "\r\n" +
	                      "    </noframes>" + "\r\n" +
                          "</html>";
				writer.Write(strText);
            }
            project.ProjectItems.AddFromFile(strFileName);
            Utility.SetProjectStartPage(m_vsApp, strPrjName, "index" + strCardName + ".html");
        }

        //*********************************************************     
        //
        /// <summary>
        /// GenFrameworkXmlFilev:        Generate the framwork xml
        /// </summary>
        /// <param name="strPrjName">    Project name</param>
        //  
        //*********************************************************     
        void GenFrameworkXmlFilev(string strPrjName) {
            string              strText;
            string              strFileName;
            StreamWriter        writer;
            EnvDTE.Project      project;
            string              strLangList = String.Empty;

            project     = GetProjectP(strPrjName);
            strFileName = Path.GetDirectoryName(project.FullName);
            strFileName = Path.Combine(strFileName, "RepPPInternetFramework.xml");
            foreach (object code in m_arrLanguages) {
                if (strLangList != String.Empty) {
                    strLangList += ",";
                }
                strLangList += code.ToString();
            }
            strText     =   "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n" +
                            "<!DOCTYPE configuration [\r\n" +
                            "\r\n" +
                            "<!ELEMENT configuration (trace,applications)>\r\n" +
                            "<!ELEMENT trace (category)*>\r\n" +
                            "<!ATTLIST trace type CDATA #FIXED \"file\"\r\n" +
                            "                name CDATA #REQUIRED>\r\n" +
                            "\r\n" +
                            "<!ELEMENT category	EMPTY>\r\n" +
                            "<!ATTLIST category	id (trcatStart | trcatFileSearch | trcatFileOpen | trcatCreateClass | trcatCallWorkingObject |  trcatAccessParameters | trcatAccessRequest | trcatAccessResponse | trcatAccessSession |trcatErrorMessage | trcatFrameworkSetup | trcatFrameworkUsage | trcatDBAccess ) #REQUIRED \r\n" +
    	 	   	            "          level (1 | 2 | 3 | 4) \"3\">\r\n" +
                            "\r\n" +
                            "<!ELEMENT applications (application)+>\r\n" +
                            "<!ELEMENT application	EMPTY>\r\n" +
                            "<!ATTLIST application	name		ID	#REQUIRED\r\n" +
			                "                       system		CDATA	#REQUIRED\r\n" +
    			            "                       program		CDATA	#REQUIRED\r\n" +
	    		            "                       connection	CDATA	#REQUIRED\r\n" +
		    	            "                       user		CDATA	#REQUIRED\r\n" +
			                "                       password	CDATA	#IMPLIED\r\n" +
			                "                       language	CDATA	#REQUIRED\r\n" +
                            "                       option      (0)     #FIXED \"0\"\r\n" +
			                "                       timeout		CDATA	\"1200\">\r\n" +
                            "]>\r\n" +
                            "<!-- \r\n" +
	                        "        default level is 3 for missing categories\r\n" +
                            "-->\r\n" +
                            "<configuration>\r\n" +
	                        "    <trace type=\"file\" name=\"c:\\temp\\FrameworkInternet.trc\">\r\n" +
		                    "        <category id=\"Dispatch\" 		level=\"3\"/>\r\n" +
    		                "        <category id=\"History\" 		level=\"3\"/>\r\n" +
	    	                "        <category id=\"Parameter\" 	level=\"3\"/>\r\n" +
		                    "        <category id=\"GenHTML\" 	    level=\"3\"/>\r\n" +
	                        "    </trace>\r\n" +
	                        "    <applications>\r\n" +
    		                "        <application name=\"" + strPrjName + 
                                              "\" system=\"" + m_app.SystemName + 
                                              "\" program=\"" + m_app.ProgramName + 
                                              "\" connection=\"" + m_app.ConnectionName + 
                                              "\" user=\"" + m_app.UserName + 
                                              "\" password=\"" +
                                              "\" language=\"" + strLangList + "\"" + 
                                              " option=\"0\" timeout=\"30000\"/>\r\n" +
	                        "    </applications>\r\n" +
                            "</configuration>";
            using (writer = Utility.CreateTextFile(strFileName, Utility.ReplaceFileModeE.sdReplaceWithoutAsk)) {
                writer.Write(strText);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// InitProject:                Initialize the project
        /// </summary>
        /// <param name="strPrjName">   Project name</param>
        //
        //*********************************************************     
        private void InitProject(string strPrjName) {
            EnvDTE.Project              project;
            VSLangProj.VSProject        vsProject;
            string                      strBinDir;
            string                      strDllName;

            project    = GetProjectP(strPrjName);
            vsProject  = project.Object as VSLangProj.VSProject;
            strBinDir  = Utility.GetRepPPBinDir();
            if (vsProject.References.Find("sdfwwebcsnet") == null) {
                strDllName = Path.Combine(strBinDir, "sdfwwebcsnet.dll");
                vsProject.References.Add(strDllName);               
            }
            if (vsProject.References.Find("sddnet") == null) {
                strDllName = Path.Combine(strBinDir, "sddnet.dll");
                vsProject.References.Add(strDllName);               
            }
            if (vsProject.References.Find("RepPP.Toolkit") == null) {
                strDllName = Path.Combine(strBinDir, "RepPP.Toolkit.V1.dll");
                vsProject.References.Add(strDllName);               
            }
            Utility.CreateSessionFolder(Path.GetDirectoryName(project.FullName));
            if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                Utility.AddAssemblyAttribute(project, 
                                             "RepPP.AssemblyIncludeTypes",
                                             CodeFileExtension);
            }
        }
        //*********************************************************     
        /// <summary>
        /// UpdateConfigFilev:          Update the configuration file (Web.config)
        /// </summary>
        /// <param name="strPrjName">   Project name</param>
        //
        //*********************************************************     
        private void UpdateConfigFilev(string strPrjName) {
            System.Xml.XmlDocument  xmlDoc;
            System.Xml.XmlNode      xmlNodeHTTP;
            System.Xml.XmlNode      xmlNodeADD;
            System.Xml.XmlAttribute xmlAttr;
            System.Xml.XmlNode      xmlNodeCONFIG;
            System.Xml.XmlNode      xmlNodeSYS;
            string                  strFileName;

            strFileName = System.IO.Path.GetDirectoryName(GetProjectP(strPrjName).FullName);
            strFileName = System.IO.Path.Combine(strFileName, "Web.config");
            if (System.IO.File.Exists(strFileName)) {
                xmlDoc = new XmlDocument();
                xmlDoc.Load(strFileName);
                xmlNodeCONFIG = Utility.FindXmlNode(xmlDoc, null, "configuration");
                xmlNodeSYS    = Utility.FindXmlNode(xmlDoc, xmlNodeCONFIG, "system.web");
                xmlNodeHTTP   = xmlDoc.CreateNode(XmlNodeType.Element, "httpHandlers", "");
                xmlNodeSYS.AppendChild(xmlNodeHTTP);
                xmlNodeADD    = xmlDoc.CreateNode(XmlNodeType.Element, "add", "");
                xmlNodeHTTP.AppendChild(xmlNodeADD);
                xmlAttr       = Utility.AddXmlNodeAttribute(xmlDoc, xmlNodeADD, "verb", "GET,POST");
                xmlAttr       = Utility.AddXmlNodeAttribute(xmlDoc, xmlNodeADD, "path", "*.rem");
                xmlAttr       = Utility.AddXmlNodeAttribute(xmlDoc, xmlNodeADD, "type", strPrjName + ".SCUserDispatch, " + strPrjName);
                xmlDoc.Save(strFileName);
            }
        }

        //**********************************************************************
        //
        /// <summary>
        /// GetProjectp:                Get the DTE project object
        /// </summary>
        /// <param name="strPrjName">   Project name</param>
        /// <returns>
        /// The project
        /// </returns>
        //
        //**********************************************************************
        protected EnvDTE.Project GetProjectP(string strPrjName) {
            return(Utility.GetProject(m_vsApp, strPrjName));
        }

        //******************************************************************
        //
        /// <summary>
        /// MySCGenTemplate:               
        /// </summary>
        //
        //******************************************************************
        private class MySCGenTemplate : SCGenTemplate {
            private EnvDTE.Project              m_project;      // Current DTE project
            private RepPP.IApplication          m_app;          // Rep+= application
            private string                      m_strCardName;  // Card name


            //********************************************************************************
            //
            /// <summary>
            /// ctor:                   Class constructor
            /// </summary>
            /// <param name="project">  DTE project</param>
            //
            //********************************************************************************            
            public MySCGenTemplate(EnvDTE.Project project, RepPP.IApplication app, string strCardName) {
                m_project       = project;
                m_app           = app;
                m_strCardName   = strCardName;
            }
            //********************************************************************************
            //
            /// <summary>
            /// CanGenerateFileB:           Check if the HTML file can be generated
            /// </summary>
            /// <param name="strFileName">  HTML file name</param>
            /// <returns>
            /// True:       The file can be generated
            /// False:      The file can not be generated
            /// </returns>
            //
            //********************************************************************************
            protected override bool CanGenerateFileB(string strFileName) {
                bool bRetVal = false;

                if (!System.IO.File.Exists(strFileName)) {
                    bRetVal = true;
                } else if (MessageBox.Show("The file " + strFileName + " already exists.\r\n" +
                                           " Do you want ro replace it?",
					                       "Rep++ Web Wizard",
					                       MessageBoxButtons.YesNo,
					                       MessageBoxIcon.Question) == DialogResult.Yes) {
                    // Delete the current file
                    System.IO.File.Delete(strFileName);
                    bRetVal = true;
                }
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// OnFileCreated:      Trigger the file created event 
            /// </summary>
            /// <param name="e">    Event arguments</param>
            //
            //********************************************************************************
            protected override void OnFileCreated(sdfwwebcsnet.SCGenTemplate.FileCreatedEventArgs e) {
                string          strFileName;
                RepPP.Card      crd;
                RepPP.Module    mod;

                base.OnFileCreated(e);
                m_project.ProjectItems.AddFromFile(e.FileName);
                strFileName = Path.GetFileName(e.FileName);
                crd         = m_app.Cards[m_strCardName];
                mod         = crd.Module;
                if (!IsHTMLFileExistInRepb(strFileName, mod.System, mod.Section, mod.Name)) {
                    InsertHTMLFileInRepv(strFileName, mod.System, mod.Section, mod.Name);
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// IsHTMLFileExistInRepb:      Check if the HTML file is registered in the repository
            /// </summary>
            /// <param name="strFileName">  File name</param>
            /// <param name="strSysName">   System name</param>
            /// <param name="strModName">   Module name</param>
            //
            //********************************************************************************
            private bool IsHTMLFileExistInRepb(string    strFileName,
                                               string    strSysName,
                                               string    strSectFullPath,
                                               string    strModName) {
                bool                bRetVal	    = true;
                RepPP.SqlCommand    sqlCommand  = null;
                RepPP.SqlCursor     sqlCursor	= null;

                try {
                    sqlCommand = m_app.DataConnection.SqlCommands.Open(IS_HTML_EXIST_CMD_SQL, RepPP.SqlOptimization.sdOptCursor);
                    if (sqlCommand != null) {
                        sqlCommand.SetParameterValue("APPSYSTNAME",    strSysName, RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("SECTFULLPATH",   strSectFullPath, RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("MODNAME",        strModName, RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("HTMLFILENAME",   strFileName, RepPP.FieldType.sdFieldString);
                        if (sqlCommand.Execute(ref sqlCursor) != 0) {
                            throw new System.Exception(m_app.DataConnection.ErrorMessage);
                        } 
                        if (sqlCursor.Next() == 0) {
                            bRetVal = Int32.Parse(sqlCursor.GetColumnValue(0)) != 0;
                        }
                    }
                } finally {
                    if (sqlCursor != null) {
                        sqlCursor.Close();
					    sqlCursor = null;
                    }
                    if (sqlCommand != null) {
                        sqlCommand.Close();
					    sqlCommand = null;
                    }
                }
                return(bRetVal);                       
            }

            //********************************************************************************
            //
            /// <summary>
            /// InsertHTMLFileInRep:            Check if the HTML file is registered in the repository
            /// </summary>
            /// <param name="strFileName">      File name</param>
            /// <param name="strSysName">       System name</param>
            /// <param name="strSectFullPath">  Section full path</param>
            /// <param name="strModName">       Module name</param>
            //
            //********************************************************************************
            private void InsertHTMLFileInRepv(string    strFileName,
                                              string    strSysName,
                                              string    strSectFullPath,
                                              string    strModName) {
                RepPP.SqlCommand    sqlCommand  = null;
                RepPP.SqlCursor     sqlCursor	= null;

                try {
                    sqlCommand = m_app.DataConnection.SqlCommands.Open(INSERT_HTML_CMD_SQL, RepPP.SqlOptimization.sdOptCursor);
                    if (sqlCommand != null) {
                        sqlCommand.SetParameterValue("APPSYSTNAME",     strSysName,      RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("SECTFULLPATH",    strSectFullPath, RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("MODNAME",         strModName,      RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("HTMLFILENAMEUC",  strFileName,     RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("HTMLFILENAME",    strFileName,     RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("HTMLOPT",         "44739242",      RepPP.FieldType.sdFieldNumeric);
                        sqlCommand.SetParameterValue("ISMULTILANGUAGE", "1",             RepPP.FieldType.sdFieldNumeric);
                        sqlCommand.SetParameterValue("HTMLFILEDESC",    "",              RepPP.FieldType.sdFieldString);
                        sqlCommand.SetParameterValue("ATTRSVAL",        "",              RepPP.FieldType.sdFieldString);
                        if (sqlCommand.Execute(ref sqlCursor) != 0) {
                            throw new System.Exception(m_app.DataConnection.ErrorMessage);
                        } 
                    }
                } finally {
                    if (sqlCursor != null) {
                        sqlCursor.Close();
					    sqlCursor = null;
                    }
                    if (sqlCommand != null) {
                        sqlCommand.Close();
					    sqlCommand = null;
                    }
                }
            }
        } // class MySCGenTemplate
    } // class GeneratorBase
} // namespace
